@@ -392,46 +392,6 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
   g_array_free (array, TRUE);
 }
 
-
-void
-ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
-                                           const char          *settings_policy)
-{
-  WebKitCookieAcceptPolicy policy;
-
-  if (!strcmp (settings_policy, "never"))
-    policy = WEBKIT_COOKIE_POLICY_ACCEPT_NEVER;
-  else if (!strcmp (settings_policy, "always"))
-    policy = WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS;
-  else if (!strcmp (settings_policy, "no-third-party"))
-    policy = WEBKIT_COOKIE_POLICY_ACCEPT_NO_THIRD_PARTY;
-  else {
-    g_warn_if_reached ();
-    return;
-  }
-
-  webkit_cookie_manager_set_accept_policy (cookie_manager, policy);
-}
-
-static void
-webkit_pref_callback_cookie_accept_policy (GSettings  *settings,
-                                           const char *key,
-                                           gpointer    data)
-{
-  WebKitCookieManager *cookie_manager;
-  char *value;
-  EphyEmbedShell *shell;
-
-  value = g_settings_get_string (settings, key);
-  if (!value)
-    return;
-
-  shell = ephy_embed_shell_get_default ();
-  cookie_manager = webkit_web_context_get_cookie_manager (ephy_embed_shell_get_web_context (shell));
-  ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, value);
-  g_free (value);
-}
-
 static void
 webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
                                   const char *key,
@@ -489,6 +449,26 @@ webkit_pref_callback_enable_spell_checking (GSettings  *settings,
   }
 }
 
+static void
+webkit_pref_callback_enable_website_data_storage (GSettings  *settings,
+                                                  const char *key,
+                                                  gpointer    data)
+{
+  EphyEmbedShell *shell;
+  WebKitWebContext *context;
+  WebKitCookieManager *manager;
+  gboolean value;
+
+  value = g_settings_get_boolean (settings, key);
+  webkit_settings_set_enable_html5_database (webkit_settings, value);
+  webkit_settings_set_enable_html5_local_storage (webkit_settings, value);
+
+  shell = ephy_embed_shell_get_default ();
+  context = ephy_embed_shell_get_web_context (shell);
+  manager = webkit_web_context_get_cookie_manager (context);
+  webkit_cookie_manager_set_accept_policy (manager, value ? WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS : WEBKIT_COOKIE_POLICY_ACCEPT_NEVER);
+}
+
 static void
 webkit_pref_callback_hardware_acceleration_policy (GSettings  *settings,
                                                    const char *key,
@@ -569,9 +549,9 @@ static const PrefData webkit_pref_entries[] = {
     "user-agent",
     webkit_pref_callback_user_agent },
   { EPHY_PREFS_WEB_SCHEMA,
-    EPHY_PREFS_WEB_COOKIES_POLICY,
-    "accept-policy",
-    webkit_pref_callback_cookie_accept_policy },
+    EPHY_PREFS_WEB_ENABLE_WEBSITE_DATA_STORAGE,
+    NULL,
+    webkit_pref_callback_enable_website_data_storage },
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY,
     "hardware-acceleration-policy",
@@ -587,8 +567,6 @@ ephy_embed_prefs_init (gpointer user_data)
                                                        "enable-developer-extras", TRUE,
                                                        "enable-dns-prefetching", TRUE,
                                                        "enable-plugins", FALSE,
-                                                       "enable-webaudio", TRUE, /* FIXME: https://bugs.webkit.org/show_bug.cgi?id=205334 */
-                                                       "enable-webgl", TRUE, /* FIXME: https://bugs.webkit.org/show_bug.cgi?id=205335 */
                                                        "javascript-can-open-windows-automatically", TRUE,
 #if WEBKIT_CHECK_VERSION (2, 27, 90) && !WEBKIT_CHECK_VERSION (2, 30, 0)
                                                        "allow-top-navigation-to-data-urls", FALSE,
