@@ -637,7 +637,7 @@ filter_load_cb (WebKitUserContentFilterStore *store,
   g_assert (self);
   g_assert (store == self->manager->store);
 
-  g_clear_object (&self->manager->wk_filter);
+  g_clear_pointer (&self->manager->wk_filter, webkit_user_content_filter_unref);
   self->manager->wk_filter = webkit_user_content_filter_store_load_finish (self->manager->store,
                                                                            result,
                                                                            &error);
@@ -1092,7 +1092,6 @@ ephy_filters_manager_get_adblock_filters_dir (EphyFiltersManager *manager)
 gboolean
 ephy_filters_manager_get_is_initialized (EphyFiltersManager *manager)
 {
-  g_return_val_if_fail (EPHY_IS_FILTERS_MANAGER (manager), FALSE);
   return manager->is_initialized;
 }
 
