@@ -216,8 +216,6 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
 {
   GdkPixbuf *snapshot, *scaled;
   int orig_width, orig_height;
-  float orig_aspect_ratio, dest_aspect_ratio;
-  int x_offset, new_width, new_height;
 
   orig_width = cairo_image_surface_get_width (surface);
   orig_height = cairo_image_surface_get_height (surface);
@@ -232,25 +230,13 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
                                       EPHY_THUMBNAIL_HEIGHT,
                                       GDK_INTERP_TILES);
   } else {
-    orig_aspect_ratio = orig_width / (float)orig_height;
-    dest_aspect_ratio = EPHY_THUMBNAIL_WIDTH / (float)EPHY_THUMBNAIL_HEIGHT;
+    gfloat width_ratio = (gfloat)EPHY_THUMBNAIL_WIDTH / (gfloat)orig_width;
+    gfloat new_height = orig_height * width_ratio;
 
-    if (orig_aspect_ratio > dest_aspect_ratio) {
-      /* Wider than taller, crop the sides. */
-      new_width = orig_height * dest_aspect_ratio;
-      new_height = orig_height;
-      x_offset = (orig_width - new_width) / 2;
-    } else {
-      /* Crop the bottom otherwise. */
-      new_width = orig_width;
-      new_height = orig_width / (float)dest_aspect_ratio;
-      x_offset = 0;
-    }
-
-    snapshot = gdk_pixbuf_get_from_surface (surface, x_offset, 0, new_width, new_height);
+    snapshot = gdk_pixbuf_get_from_surface (surface, 0, 0, orig_width, orig_height);
     scaled = gdk_pixbuf_scale_simple (snapshot,
                                       EPHY_THUMBNAIL_WIDTH,
-                                      EPHY_THUMBNAIL_HEIGHT,
+                                      new_height,
                                       GDK_INTERP_BILINEAR);
   }
 
@@ -454,7 +440,7 @@ retrieve_snapshot_from_web_view (GTask *task)
   }
 
   webkit_web_view_get_snapshot (data->web_view,
-                                WEBKIT_SNAPSHOT_REGION_VISIBLE,
+                                WEBKIT_SNAPSHOT_REGION_FULL_DOCUMENT,
                                 WEBKIT_SNAPSHOT_OPTIONS_NONE,
                                 g_task_get_cancellable (task),
                                 (GAsyncReadyCallback)on_snapshot_ready,
@@ -676,6 +662,7 @@ got_snapshot_path_for_url (EphySnapshotService *service,
                            GAsyncResult        *result,
                            GTask               *task)
 {
+#ifndef __clang_analyzer__
   SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
@@ -687,6 +674,7 @@ got_snapshot_path_for_url (EphySnapshotService *service,
   } else {
     ephy_snapshot_service_take_from_webview (task);
   }
+#endif
 }
 
 void
