@@ -110,9 +110,16 @@ icon_loaded_cb (GObject      *source,
   }
 
   if (favicon) {
-    gtk_image_set_from_pixbuf (GTK_IMAGE (image), favicon);
+    cairo_surface_t *surface;
+
+    surface = gdk_cairo_surface_create_from_pixbuf (favicon,
+                                                    0,
+                                                    gtk_widget_get_window (GTK_WIDGET (image)));
+    gtk_image_set_from_surface (GTK_IMAGE (image), surface);
+
     gtk_widget_show (image);
 
+    cairo_surface_destroy (surface);
     g_object_unref (favicon);
   }
 
@@ -470,7 +477,10 @@ homepage_url_changed (GSettings  *settings,
   gboolean show_button;
 
   setting = g_settings_get_string (settings, key);
-  show_button = setting && setting[0] && g_strcmp0 (setting, "about:blank") != 0;
+  if (setting && setting[0])
+    show_button = g_strcmp0 (setting, "about:blank") != 0;
+  else
+    show_button = is_desktop_pantheon ();
 
   gtk_widget_set_visible (button, show_button);
   g_free (setting);
