@@ -70,6 +70,7 @@ struct _EphyShell {
 
   GHashTable *notifications;
   gchar *open_notification_id;
+  gboolean startup_finished;
 };
 
 static EphyShell *ephy_shell = NULL;
@@ -165,6 +166,8 @@ ephy_shell_startup_continue (EphyShell               *shell,
     EphyWindow *window = ephy_window_new ();
     ephy_link_open (EPHY_LINK (window), NULL, NULL, EPHY_LINK_HOME_PAGE);
   }
+
+  shell->startup_finished = TRUE;
 }
 
 static void
@@ -511,7 +514,7 @@ ephy_shell_startup (GApplication *application)
     set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
     set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
     set_accel_for_action (shell, "app.export-bookmarks", "<Primary><Shift>x");
-    set_accel_for_action (shell, "app.shortcuts", "<Primary>F1");
+    set_accel_for_action (shell, "app.shortcuts", "<Primary>question");
     set_accel_for_action (shell, "app.help", "F1");
   } else {
     g_action_map_add_action_entries (G_ACTION_MAP (application),
@@ -586,12 +589,33 @@ session_load_cb (GObject      *object,
   ephy_shell_startup_continue (ephy_shell_get_default (), ctx);
 }
 
+static void
+portal_check (EphyShell *shell)
+{
+  if (g_network_monitor_get_connectivity (ephy_shell_get_net_monitor (shell)) == G_NETWORK_CONNECTIVITY_PORTAL) {
+    EphyWindow *window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
+
+    ephy_window_load_url (window, "http://nmcheck.gnome.org/");
+  }
+}
+
+static void
+connectivity_changed (GNetworkMonitor *monitor,
+                      GParamSpec      *pspec,
+                      EphyShell       *shell)
+{
+  portal_check (shell);
+}
+
 static void
 ephy_shell_activate (GApplication *application)
 {
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (shell);
 
+  g_signal_connect (ephy_shell_get_net_monitor (shell), "notify::connectivity", G_CALLBACK (connectivity_changed), shell);
+  portal_check (shell);
+
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_AUTOMATION) {
     WebKitWebContext *web_context = ephy_embed_shell_get_web_context (embed_shell);
     g_signal_connect (web_context, "automation-started", G_CALLBACK (automation_started_cb), shell);
@@ -803,6 +827,7 @@ ephy_shell_init (EphyShell *shell)
   /* globally accessible singleton */
   g_assert (ephy_shell == NULL);
   ephy_shell = shell;
+  ephy_shell->startup_finished = FALSE;
   g_object_add_weak_pointer (G_OBJECT (ephy_shell),
                              (gpointer *)ptr);
 
@@ -1462,3 +1487,9 @@ ephy_shell_send_notification (EphyShell     *shell,
   shell->open_notification_id = g_strdup (id);
   g_application_send_notification (G_APPLICATION (shell), id, notification);
 }
+
+gboolean
+ephy_shell_startup_finished (EphyShell *shell)
+{
+  return shell->startup_finished;
+}
